﻿if select(2, UnitClass("player")) ~= "PALADIN" then return end

local SpellName, _, SpellIcon = GetSpellInfo(20066) -- Покаяние

local extraOptions = {
 {
  [0] = "Additional",
  { type = "Check", var = "colGradientTime", default = true, label = "Use Gradient Color for Time", tip = "", y = 12 },
  { type = "Check", var = "colGradient", default = true, label = "Use Gradient Color for Bar", tip = "", y = 8 },
  { type = "Color", var = "colNormal", default = { 0.4, 0.6, 0.8, 1.0 }, label = "Bar Color", y = 8 },
  { type = "Check", var = "colClassColor", default = true, label = "Use Class Color", tip = "", y = 12 },
  { type = "Check", var = "AbilityName", default = false, label = "Use Ability Name", tip = "", y = 12 },
 },
}
local bcr, bcg, bcb = 0.4, 0.6, 0.8
local tcr, tcg, tcb = 0, 1, 0
local useGradient, useGradientTime, useClassColor, useAbilityName = true, true, true, false

local select = select
-- http://www.wowwiki.com/ColorGradient
local function ColorGradient(perc, ...)
 if perc >= 1 then
  local r, g, b = select(select('#', ...) - 2, ...)
  return r, g, b
 elseif perc <= 0 then
  local r, g, b = ...
  return r, g, b
 end
 local num = select('#', ...) / 3
 local segment, relperc = math.modf(perc*(num-1))
 local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)
 return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

local RAID_CLASS_COLORS = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS
local sformat = string.format
local function ColorClass(unit, name)
 local c = RAID_CLASS_COLORS[select(2, UnitClass(unit)) or "PRIEST"]
 return sformat("|c%s%s|r", c.colorStr, name)
end

-- Plugin
local plugin = AzCastBar.CreateMainBar("Frame", "Repentance", extraOptions)

-- Init Bar
plugin.icon:SetTexture(SpellIcon)
plugin.name:SetText(SpellName)
plugin.name:SetJustifyV("CENTER")
plugin:SetPoint("CENTER", -50)
plugin.status:SetMinMaxValues(0, 60)
plugin.status:SetValue(0)
plugin.timeleft = 0
plugin.duration = 60
plugin.guid = nil

-------------------------------------------------------------------------------
-- Code
-------------------------------------------------------------------------------
function plugin:OnConfigChanged(cfg)
 bcr, bcg, bcb = unpack(cfg.colNormal)
 tcr, tcg, tcb = unpack(cfg.colTimeLabel)
 useGradient = cfg.colGradient
 if not useGradient then
  self.status:SetStatusBarColor(bcr, bcg, bcb, 1)
 end
 useGradientTime = cfg.colGradientTime
 if not useGradientTime then
  self.time:SetTextColor(tcr, tcg, tcb, 1)
 end
 useClassColor = cfg.colClassColor
 useAbilityName = cfg.AbilityName
 if cfg.enabled then
  self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
 else
  self:UnregisterAllEvents()
  self:Hide()
 end
end

plugin:SetScript("OnUpdate", function(self, elapsed)
 self.timeleft = self.timeleft - elapsed
 if self.timeleft > 0 then
  self.status:SetValue(self.timeleft)
  self.time:SetFormattedText("%d", ceil(self.timeleft))
  local percent = self.timeleft/self.duration
  local cr, cg, cb
  if useGradient then
   cr, cg, cb = ColorGradient(percent, 1, 0, 0, bcr, bcg, bcb)
   self.status:SetStatusBarColor(cr, cg, cb, 1)
  end
  if useGradientTime then
   cr, cg, cb = ColorGradient(percent, 1, 0, 0, tcr, tcg, tcb)
   self.time:SetTextColor(cr, cg, cb, 1)
  end
 else
  self:Hide()
 end
end)

plugin:SetScript("OnEvent", function(self, event, ...)
 local _, eventType, _, srcGUID, _, _, _, dstGUID, dstName, _, _, spellID, spellName = ...
 if spellID == 20066 and srcGUID == UnitGUID("player") then
  if eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH" then
   local WhatID = (dstGUID == UnitGUID("target")) and "target" or ((dstGUID == UnitGUID("focus")) and "focus" or ((dstGUID == UnitGUID("mouseover")) and "mouseover" or nil))
   local duration, timeleft = 60, 60
   if WhatID then
    local dur, lefttime, caster, spellid
    local index = 1
    while UnitGUID(WhatID) == dstGUID do
     dur, lefttime, caster, _, _, spellid = select(6, UnitDebuff(WhatID, index, "PLAYER"))
     if not spellid then
      break
     elseif spellid == spellID and caster then
      duration = dur
      timeleft = lefttime - GetTime()
      break
     end
     index = index + 1
    end
   end
   if self.duration ~= duration then
    self.status:SetMinMaxValues(0, duration)
    self.duration = duration
   end
   self.guid = dstGUID
   local Sure = (WhatID and UnitGUID(WhatID) == dstGUID)
   self.name:SetFormattedText("%s%s%s", Sure and "" or "~", useClassColor and ColorClass(Sure and WhatID or dstName, dstName) or dstName, useAbilityName and sformat(" (%s)", spellName) or "")
   self.timeleft = timeleft
   if not self:IsShown() then
    self:Show()
   end
  elseif eventType == "SPELL_AURA_REMOVED" and dstGUID == self.guid then
   self.timeleft = 0
   self.guid = nil
  end
 end
end)
